@@ -143,12 +143,13 @@ mapi_folder_search_by_uids (CamelFolder *folder,
 }
 
 static void
-mapi_set_message_id (CamelMapiMessageInfo *mapi_mi, const gchar *message_id)
+mapi_set_message_id (CamelMessageInfo *mi,
+		     const gchar *message_id)
 {
 	gchar *msgid;
 	guint8 *digest;
 	gsize length;
-	CamelMessageInfoBase *mi = &mapi_mi->info;
+	CamelSummaryMessageID tmp_msgid;
 
 	msgid = camel_header_msgid_decode (message_id);
 	if (msgid) {
@@ -162,24 +163,28 @@ mapi_set_message_id (CamelMapiMessageInfo *mapi_mi, const gchar *message_id)
 		g_checksum_get_digest (checksum, digest, &length);
 		g_checksum_free (checksum);
 
-		memcpy(mi->message_id.id.hash, digest, sizeof(mi->message_id.id.hash));
-		g_free(msgid);
-	}
+		memcpy (tmp_msgid.id.hash, digest, sizeof (tmp_msgid.id.hash));
+		g_free (msgid);
 
+		camel_message_info_set_message_id (mi, tmp_msgid.id.id);
+	}
 }
 
 static void
-mapi_set_message_references (CamelMapiMessageInfo *mapi_mi, const gchar *references, const gchar *in_reply_to)
+mapi_set_message_references (CamelMessageInfo *mi,
+			     const gchar *references,
+			     const gchar *in_reply_to)
 {
-	GSList *refs, *irt, *scan;
+	GSList *refs, *irt, *link;
 	guint8 *digest;
-	gint count;
 	gsize length;
-	CamelMessageInfoBase *mi = &mapi_mi->info;
+	CamelSummaryMessageID tmp_msgid;
 
 	refs = camel_header_references_decode (references);
 	irt = camel_header_references_decode (in_reply_to);
 	if (refs || irt) {
+		GArray *references;
+
 		if (irt) {
 			/* The References field is populated from the "References" and/or "In-Reply-To"
 			   headers. If both headers exist, take the first thing in the In-Reply-To header
@@ -188,28 +193,27 @@ mapi_set_message_references (CamelMapiMessageInfo *mapi_mi, const gchar *referen
 			refs = g_slist_concat (irt, refs);
 		}
 
-		count = g_slist_length (refs);
-		mi->references = g_malloc(sizeof(*mi->references) + ((count-1) * sizeof(mi->references->references[0])));
+		references = g_array_sized_new (FALSE, FALSE, sizeof (guint64), g_slist_length (refs));
 
 		length = g_checksum_type_get_length (G_CHECKSUM_MD5);
 		digest = g_alloca (length);
 
-		count = 0;
-		scan = refs;
-		while (scan) {
+		for (link = refs; link; link = g_slist_next (link)) {
 			GChecksum *checksum;
 
 			checksum = g_checksum_new (G_CHECKSUM_MD5);
-			g_checksum_update (checksum, (guchar *) scan->data, -1);
+			g_checksum_update (checksum, (guchar *) link->data, -1);
 			g_checksum_get_digest (checksum, digest, &length);
 			g_checksum_free (checksum);
 
-			memcpy(mi->references->references[count].id.hash, digest, sizeof(mi->message_id.id.hash));
-			count++;
-			scan = g_slist_next (scan);
+			memcpy (tmp_msgid.id.hash, digest, sizeof (tmp_msgid.id.hash));
+
+			g_array_append_val (references, tmp_msgid.id.id);
 		}
-		mi->references->size = count;
+
 		g_slist_free_full (refs, g_free);
+
+		camel_message_info_take_references (mi, references);
 	}
 }
 
@@ -233,7 +237,7 @@ add_message_to_cache (CamelMapiFolder *mapi_folder, const gchar *uid, CamelMimeM
 	folder = CAMEL_FOLDER (mapi_folder);
 	g_return_if_fail (folder != NULL);
 
-	camel_folder_summary_lock (folder->summary);
+	camel_folder_summary_lock (camel_folder_get_folder_summary (folder));
 
 	base_stream = camel_data_cache_add (mapi_folder->cache, "cache", uid, NULL);
 	if (base_stream != NULL) {
@@ -265,7 +269,7 @@ add_message_to_cache (CamelMapiFolder *mapi_folder, const gchar *uid, CamelMimeM
 		g_object_unref (cache_stream);
 	}
 
-	camel_folder_summary_unlock (folder->summary);
+	camel_folder_summary_unlock (camel_folder_get_folder_summary (folder));
 }
 
 struct GatherChangedObjectsData
@@ -307,9 +311,9 @@ gather_changed_objects_to_slist (EMapiConnection *conn,
 
 		info = camel_folder_summary_get (gco->summary, uid_str);
 		if (info) {
-			CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
+			CamelMapiMessageInfo *minfo = CAMEL_MAPI_MESSAGE_INFO (info);
 
-			if (minfo->last_modified != object_data->last_modified
+			if (camel_mapi_message_info_get_last_modified (minfo) != object_data->last_modified
 			    && (object_data->msg_flags & MSGFLAG_UNMODIFIED) == 0) {
 				update = TRUE;
 			} else {
@@ -324,26 +328,20 @@ gather_changed_objects_to_slist (EMapiConnection *conn,
 				if ((object_data->msg_flags & MSGFLAG_HASATTACH) != 0)
 					flags |= CAMEL_MESSAGE_ATTACHMENTS;
 
-				if ((minfo->info.flags & CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT) != 0) {
+				if ((camel_message_info_get_flags (info) & CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT) != 0) {
 					if ((object_data->msg_flags & MSGFLAG_RN_PENDING) == 0 &&
 					    !camel_message_info_get_user_flag (info, "receipt-handled")) {
 						camel_message_info_set_user_flag (info, "receipt-handled", TRUE);
-						minfo->info.dirty = TRUE;
-
-						camel_folder_summary_touch (gco->summary);
 					}
 				}
 
-				if ((minfo->info.flags & mask) != (flags & mask)) {
+				if ((camel_message_info_get_flags (info) & mask) != (flags & mask)) {
 					camel_message_info_set_flags (info, mask, flags);
-					minfo->server_flags = camel_message_info_get_flags (info);
-					minfo->info.dirty = TRUE;
-
-					camel_folder_summary_touch (gco->summary);
+					camel_mapi_message_info_set_server_flags (minfo, camel_message_info_get_flags (info));
 				}
 			}
 
-			camel_message_info_unref (info);
+			g_clear_object (&info);
 		}
 	} else {
 		update = TRUE;
@@ -375,7 +373,6 @@ update_message_info (CamelMessageInfo *info,
 		     gboolean is_public_folder,
 		     gboolean user_has_read)
 {
-	CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
 	guint32 flags = 0, mask = CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_ATTACHMENTS | CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_FORWARDED | CAMEL_MAPI_MESSAGE_WITH_READ_RECEIPT;
 	const uint32_t *pmsg_flags, *picon_index;
 	const struct FILETIME *last_modified;
@@ -392,11 +389,11 @@ update_message_info (CamelMessageInfo *info,
 	pread_receipt = e_mapi_util_find_array_propval (&object->properties, PidTagReadReceiptRequested);
 	msg_class = e_mapi_util_find_array_propval (&object->properties, PidTagMessageClass);
 
-	if (!minfo->info.size) {
+	if (!camel_message_info_get_size (info)) {
 		const uint32_t *msg_size;
 
 		msg_size = e_mapi_util_find_array_propval (&object->properties, PidTagMessageSize);
-		minfo->info.size = msg_size ? *msg_size : 0;
+		camel_message_info_set_size (info, msg_size ? *msg_size : 0);
 	}
 
 	if (msg_class && g_str_has_prefix (msg_class, "REPORT.IPM.Note.IPNRN"))
@@ -410,11 +407,8 @@ update_message_info (CamelMessageInfo *info,
 			msg_flags = (msg_flags & (~MSGFLAG_READ)) | (user_has_read ? MSGFLAG_READ : 0);
 	}
 
-	if (last_modified) {
-		minfo->last_modified = e_mapi_util_filetime_to_time_t (last_modified);
-	} else {
-		minfo->last_modified = 0;
-	}
+	camel_mapi_message_info_set_last_modified (CAMEL_MAPI_MESSAGE_INFO (info),
+		last_modified ? e_mapi_util_filetime_to_time_t (last_modified) : 0);
 
 	if ((msg_flags & MSGFLAG_READ) != 0)
 		flags |= CAMEL_MESSAGE_SEEN;
@@ -435,34 +429,21 @@ update_message_info (CamelMessageInfo *info,
 
 	if ((camel_message_info_get_flags (info) & mask) != flags) {
 		if (is_new)
-			minfo->info.flags = flags;
+			camel_message_info_set_flags (info, ~0, flags);
 		else
 			camel_message_info_set_flags (info, mask, flags);
-		minfo->server_flags = camel_message_info_get_flags (info);
+		camel_mapi_message_info_set_server_flags (CAMEL_MAPI_MESSAGE_INFO (info), camel_message_info_get_flags (info));
 	}
-
-	minfo->info.dirty = TRUE;
-	camel_folder_summary_touch (minfo->info.summary);
 }
 
 static gsize
 camel_mapi_get_message_size (CamelMimeMessage *msg)
 {
-	CamelStream *null;
-	CamelDataWrapper *dw;
-	gsize sz;
-
 	if (!CAMEL_IS_DATA_WRAPPER (msg))
 		return 0;
 
-	dw = CAMEL_DATA_WRAPPER (msg);
-	null = camel_stream_null_new ();
 	/* do not 'decode', let's be interested in the raw message size */
-	camel_data_wrapper_write_to_stream_sync (dw, null, NULL, NULL);
-	sz = CAMEL_STREAM_NULL (null)->written;
-	g_object_unref (null);
-
-	return sz;
+	return camel_data_wrapper_calculate_size_sync (CAMEL_DATA_WRAPPER (msg), NULL, NULL);
 }
 
 struct GatherObjectSummaryData
@@ -482,7 +463,7 @@ remove_removed_uids_cb (gpointer uid_str, gpointer value, gpointer user_data)
 	g_return_if_fail (gos->changes != NULL);
 
 	camel_folder_change_info_remove_uid (gos->changes, uid_str);
-	camel_folder_summary_remove_uid (gos->folder->summary, uid_str);
+	camel_folder_summary_remove_uid (camel_folder_get_folder_summary (gos->folder), uid_str);
 	camel_data_cache_remove (CAMEL_MAPI_FOLDER (gos->folder)->cache, "cache", uid_str, NULL);
 }
 
@@ -505,6 +486,7 @@ gather_object_for_offline_cb (EMapiConnection *conn,
 
 	msg = e_mapi_mail_utils_object_to_message (conn, object);
 	if (msg) {
+		CamelFolderSummary *folder_summary;
 		gchar *uid_str;
 		const mapi_id_t *pmid;
 		CamelMessageInfo *info;
@@ -528,44 +510,45 @@ gather_object_for_offline_cb (EMapiConnection *conn,
 		if (!uid_str)
 			return FALSE;
 
-		is_new = !camel_folder_summary_check_uid (gos->folder->summary, uid_str);
+		folder_summary = camel_folder_get_folder_summary (gos->folder);
+		is_new = !camel_folder_summary_check_uid (folder_summary, uid_str);
 		if (!is_new) {
 			/* keep local read/unread flag on messages from public folders */
 			if (gos->is_public_folder) {
-				info = camel_folder_summary_get (gos->folder->summary, uid_str);
+				info = camel_folder_summary_get (folder_summary, uid_str);
 				if (info) {
 					user_has_read = (camel_message_info_get_flags (info) & CAMEL_MESSAGE_SEEN) != 0;
-					camel_message_info_unref (info);
+					g_clear_object (&info);
 				}
 			}
 
-			camel_folder_summary_remove_uid (gos->folder->summary, uid_str);
+			camel_folder_summary_remove_uid (folder_summary, uid_str);
 		}
 
-		info = camel_folder_summary_info_new_from_message (gos->folder->summary, msg, NULL);
+		info = camel_folder_summary_info_new_from_message (folder_summary, msg);
 		if (info) {
-			CamelMapiMessageInfo *minfo = (CamelMapiMessageInfo *) info;
+			camel_message_info_set_abort_notifications (info, TRUE);
 
-			minfo->info.uid = camel_pstring_strdup (uid_str);
+			camel_message_info_set_uid (info, uid_str);
 
 			update_message_info (info, object, is_new, gos->is_public_folder, user_has_read);
 
-			if (!minfo->info.size)
-				minfo->info.size = camel_mapi_get_message_size (msg);
+			if (!camel_message_info_get_size (info))
+				camel_message_info_set_size (info, camel_mapi_get_message_size (msg));
 
-			camel_folder_summary_add (gos->folder->summary, info);
-			camel_message_info_ref (info);
+			camel_message_info_set_abort_notifications (info, FALSE);
+			camel_folder_summary_add (folder_summary, info, FALSE);
 
 			if (is_new) {
-				camel_folder_change_info_add_uid (gos->changes, camel_message_info_get_uid (info));
-				camel_folder_change_info_recent_uid (gos->changes, camel_message_info_get_uid (info));
+				camel_folder_change_info_add_uid (gos->changes, uid_str);
+				camel_folder_change_info_recent_uid (gos->changes, uid_str);
 			} else {
-				camel_folder_change_info_change_uid (gos->changes, camel_message_info_get_uid (info));
+				camel_folder_change_info_change_uid (gos->changes, uid_str);
 			}
 
 			add_message_to_cache (CAMEL_MAPI_FOLDER (gos->folder), uid_str, &msg, cancellable);
 
-			camel_message_info_unref (info);
+			g_clear_object (&info);
 		} else {
 			g_debug ("%s: Failed to create message info from message", G_STRFUNC);
 		}
@@ -622,10 +605,8 @@ gather_object_summary_cb (EMapiConnection *conn,
 	if (!uid_str)
 		return FALSE;
 
-	info = camel_folder_summary_get (gos->folder->summary, uid_str);
+	info = camel_folder_summary_get (camel_folder_get_folder_summary (gos->folder), uid_str);
 	if (!info) {
-		CamelMapiMessageInfo *minfo;
-
 		is_new = TRUE;
 
 		if (transport_headers && *transport_headers) {
@@ -640,15 +621,17 @@ gather_object_summary_cb (EMapiConnection *conn,
 			g_object_unref (stream);
 
 			if (camel_mime_part_construct_from_parser_sync (part, parser, NULL, NULL)) {
-				info = camel_folder_summary_info_new_from_header (gos->folder->summary, part->headers);
+				info = camel_folder_summary_info_new_from_headers (
+					camel_folder_get_folder_summary (gos->folder),
+					camel_medium_get_headers (CAMEL_MEDIUM (part)));
 				if (info) {
 					const uint32_t *msg_size;
 
-					minfo = (CamelMapiMessageInfo *) info;
-					minfo->info.uid = camel_pstring_strdup (uid_str);
+					camel_message_info_freeze_notifications (info);
+					camel_message_info_set_uid (info, uid_str);
 
 					msg_size = e_mapi_util_find_array_propval (&object->properties, PidTagMessageSize);
-					minfo->info.size = msg_size ? *msg_size : 0;
+					camel_message_info_set_size (info, msg_size ? *msg_size : 0);
 				}
 			}
 
@@ -673,19 +656,20 @@ gather_object_summary_cb (EMapiConnection *conn,
 			display_to = e_mapi_util_find_array_propval (&object->properties, PidTagDisplayTo);
 			display_cc = e_mapi_util_find_array_propval (&object->properties, PidTagDisplayCc);
 
-			info = camel_message_info_new (gos->folder->summary);
-			minfo = (CamelMapiMessageInfo *) info;
+			info = camel_message_info_new (camel_folder_get_folder_summary (gos->folder));
 
-			minfo->info.uid = camel_pstring_strdup (uid_str);
-			minfo->info.subject = camel_pstring_strdup (subject);
-			minfo->info.date_sent = e_mapi_util_filetime_to_time_t (submit_time);
-			minfo->info.date_received = e_mapi_util_filetime_to_time_t (delivery_time);
-			minfo->info.size = msg_size ? *msg_size : 0;
+			camel_message_info_freeze_notifications (info);
+
+			camel_message_info_set_uid (info, uid_str);
+			camel_message_info_set_subject (info, subject);
+			camel_message_info_set_date_sent (info, e_mapi_util_filetime_to_time_t (submit_time));
+			camel_message_info_set_date_received (info, e_mapi_util_filetime_to_time_t (delivery_time));
+			camel_message_info_set_size (info, msg_size ? *msg_size : 0);
 
 			/* Threading related properties */
-			mapi_set_message_id (minfo, message_id);
+			mapi_set_message_id (info, message_id);
 			if (references || in_reply_to)
-				mapi_set_message_references (minfo, references, in_reply_to);
+				mapi_set_message_references (info, references, in_reply_to);
 
 			/* Recipients */
 			to_addr = (CamelAddress *) camel_internet_address_new ();
@@ -696,18 +680,18 @@ gather_object_summary_cb (EMapiConnection *conn,
 
 			if (camel_address_length (to_addr) > 0) {
 				formatted_addr = camel_address_format (to_addr);
-				minfo->info.to = camel_pstring_strdup (formatted_addr);
+				camel_message_info_set_to (info, formatted_addr);
 				g_free (formatted_addr);
 			} else {
-				minfo->info.to = camel_pstring_strdup (display_to);
+				camel_message_info_set_to (info, display_to);
 			}
 
 			if (camel_address_length (cc_addr) > 0) {
 				formatted_addr = camel_address_format (cc_addr);
-				minfo->info.cc = camel_pstring_strdup (formatted_addr);
+				camel_message_info_set_cc (info, formatted_addr);
 				g_free (formatted_addr);
 			} else {
-				minfo->info.cc = camel_pstring_strdup (display_cc);
+				camel_message_info_set_cc (info, display_cc);
 			}
 
 			g_object_unref (to_addr);
@@ -726,7 +710,7 @@ gather_object_summary_cb (EMapiConnection *conn,
 			if (from_email && *from_email) {
 				formatted_addr = camel_internet_address_format_address (from_name, from_email);
 
-				minfo->info.from = camel_pstring_strdup (formatted_addr);
+				camel_message_info_set_from (info, formatted_addr);
 
 				g_free (formatted_addr);
 			}
@@ -735,28 +719,29 @@ gather_object_summary_cb (EMapiConnection *conn,
 			g_free (from_email);
 		}
 
-		minfo = (CamelMapiMessageInfo *) info;
-		if (minfo->info.date_sent == 0)
-			minfo->info.date_sent = minfo->info.date_received;
-		if (minfo->info.date_received == 0)
-			minfo->info.date_received = minfo->info.date_sent;
+		if (!camel_message_info_get_date_sent (info))
+			camel_message_info_set_date_sent (info, camel_message_info_get_date_received (info));
+		if (!camel_message_info_get_date_received (info))
+			camel_message_info_set_date_received (info, camel_message_info_get_date_sent (info));
+	} else {
+		camel_message_info_freeze_notifications (info);
 	}
 
 	user_has_read = (camel_message_info_get_flags (info) & CAMEL_MESSAGE_SEEN) != 0;
 
 	update_message_info (info, object, is_new, gos->is_public_folder, user_has_read);
 
+	camel_message_info_thaw_notifications (info);
+
 	if (is_new) {
-		camel_folder_summary_add (gos->folder->summary, info);
+		camel_folder_summary_add (camel_folder_get_folder_summary (gos->folder), info, FALSE);
 		camel_folder_change_info_add_uid (gos->changes, camel_message_info_get_uid (info));
 		camel_folder_change_info_recent_uid (gos->changes, camel_message_info_get_uid (info));
-
-		camel_message_info_ref (info);
 	} else {
 		camel_folder_change_info_change_uid (gos->changes, camel_message_info_get_uid (info));
 	}
 
-	camel_message_info_unref (info);
+	g_clear_object (&info);
 
 	if (obj_total > 0)
 		camel_operation_progress (cancellable, obj_index * 100 / obj_total);
@@ -816,7 +801,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 
 	gco.latest_last_modify = 0;
 	gco.fid = mapi_object_get_id (&obj_folder);
-	gco.summary = folder->summary;
+	gco.summary = camel_folder_get_folder_summary (folder);
 	gco.to_update = NULL;
 	gco.removed_uids = NULL;
 	gco.is_public_folder = (mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0;
@@ -824,10 +809,10 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	if (msi->latest_last_modify <= 0) {
 		GPtrArray *known_uids;
 
-		camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
+		camel_folder_summary_prepare_fetch_all (camel_folder_get_folder_summary (folder), NULL);
 
 		gco.removed_uids = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify) camel_pstring_free, NULL);
-		known_uids = camel_folder_summary_get_array (folder->summary);
+		known_uids = camel_folder_summary_get_array (camel_folder_get_folder_summary (folder));
 		if (known_uids) {
 			gint ii;
 
@@ -901,7 +886,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	if (mapi_error && *mapi_error)
 		camel_mapi_store_maybe_disconnect (mapi_store, *mapi_error);
 
-	camel_folder_summary_save_to_db (folder->summary, NULL);
+	camel_folder_summary_save (camel_folder_get_folder_summary (folder), NULL);
 	camel_folder_thaw (folder);
 
 	return status;
@@ -983,7 +968,7 @@ mapi_refresh_folder (CamelFolder *folder, GCancellable *cancellable, GError **er
 		goto end1;
 	}
 
-	camel_folder_summary_touch (folder->summary);
+	camel_folder_summary_touch (camel_folder_get_folder_summary (folder));
 
 end1:
 	return success;
@@ -1004,44 +989,16 @@ mapi_folder_search_free (CamelFolder *folder, GPtrArray *uids)
 
 }
 
-#if 0
-static CamelMessageInfo*
-mapi_get_message_info(CamelFolder *folder, const gchar *uid)
+static guint32
+mapi_folder_get_permanent_flags (CamelFolder *folder)
 {
-	CamelMessageInfo	*msg_info = NULL;
-	CamelMessageInfoBase	*mi = (CamelMessageInfoBase *)msg;
-	gint			status = 0;
-	oc_message_headers_t	headers;
-
-	if (folder->summary) {
-		msg_info = camel_folder_summary_uid(folder->summary, uid);
-	}
-	if (msg_info != NULL) {
-		mi = (CamelMessageInfoBase *)msg_info;
-		return (msg_info);
-	}
-	/* Go online and fetch message summary. */
-
-	msg_info = camel_message_info_new(folder->summary);
-	mi = (CamelMessageInfoBase *)msg_info;
-
-	if (headers.subject) mi->subject = (gchar *)camel_pstring_strdup(headers.subject);
-	if (headers.from) mi->from = (gchar *)camel_pstring_strdup(headers.from);
-	if (headers.to) mi->to = (gchar *)camel_pstring_strdup(headers.to);
-	if (headers.cc) mi->cc = (gchar *)camel_pstring_strdup(headers.cc);
-	mi->flags = headers.flags;
-
-	mi->user_flags = NULL;
-	mi->user_tags = NULL;
-	mi->date_received = 0;
-	mi->date_sent = headers.send;
-	mi->content = NULL;
-	mi->summary = folder->summary;
-	if (uid) mi->uid = g_strdup(uid);
-	oc_message_headers_release(&headers);
-	return (msg);
+	return CAMEL_MESSAGE_ANSWERED |
+		CAMEL_MESSAGE_DELETED |
+		CAMEL_MESSAGE_DRAFT |
+		CAMEL_MESSAGE_FLAGGED |
+		CAMEL_MESSAGE_SEEN |
+		CAMEL_MESSAGE_JUNK;
 }
-#endif
 
 static void
 mapi_folder_rename (CamelFolder *folder, const gchar *new)
@@ -1052,13 +1009,13 @@ mapi_folder_rename (CamelFolder *folder, const gchar *new)
 
 	camel_store_summary_disconnect_folder_summary (
 		((CamelMapiStore *) parent_store)->summary,
-		folder->summary);
+		camel_folder_get_folder_summary (folder));
 
 	((CamelFolderClass *)camel_mapi_folder_parent_class)->rename(folder, new);
 
 	camel_store_summary_connect_folder_summary (
 		((CamelMapiStore *) parent_store)->summary,
-		camel_folder_get_full_name (folder), folder->summary);
+		camel_folder_get_full_name (folder), camel_folder_get_folder_summary (folder));
 }
 
 static gint
@@ -1070,27 +1027,6 @@ mapi_cmp_uids (CamelFolder *folder, const gchar *uid1, const gchar *uid2)
 	return strcmp (uid1, uid2);
 }
 
-static gboolean
-mapi_set_message_flags (CamelFolder *folder,
-                        const gchar *uid,
-                        CamelMessageFlags flags,
-                        CamelMessageFlags set)
-{
-	CamelMessageInfo *info;
-	gint res;
-
-	g_return_val_if_fail (folder->summary != NULL, FALSE);
-
-	info = camel_folder_summary_get (folder->summary, uid);
-	if (info == NULL)
-		return FALSE;
-
-	res = camel_message_info_set_flags (info, flags, set);
-
-	camel_message_info_unref (info);
-	return res;
-}
-
 static void
 mapi_folder_dispose (GObject *object)
 {
@@ -1098,7 +1034,7 @@ mapi_folder_dispose (GObject *object)
 	CamelFolder *folder = CAMEL_FOLDER (object);
 	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER (object);
 
-	camel_folder_summary_save_to_db (folder->summary, NULL);
+	camel_folder_summary_save (camel_folder_get_folder_summary (folder), NULL);
 
 	if (mapi_folder->cache != NULL) {
 		g_object_unref (mapi_folder->cache);
@@ -1114,7 +1050,7 @@ mapi_folder_dispose (GObject *object)
 	if (parent_store) {
 		camel_store_summary_disconnect_folder_summary (
 			(CamelStoreSummary *) ((CamelMapiStore *) parent_store)->summary,
-			CAMEL_FOLDER (mapi_folder)->summary);
+			camel_folder_get_folder_summary (CAMEL_FOLDER (mapi_folder)));
 	}
 
 	/* Chain up to parent's dispose() method. */
@@ -1291,9 +1227,9 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 {
 	CamelMapiStore *mapi_store;
 	CamelMapiFolder *mapi_folder;
-	CamelMapiMessageInfo *minfo;
 	CamelMessageInfo *info;
 	CamelFolderChangeInfo *changes;
+	CamelFolderSummary *folder_summary;
 	CamelStore *parent_store;
 	GPtrArray *known_uids;
 	gint i;
@@ -1306,6 +1242,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 	deleted_items_uid = deleted_items_uid_head = NULL;
 
 	parent_store = camel_folder_get_parent_store (folder);
+	folder_summary = camel_folder_get_folder_summary (folder);
 
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
@@ -1321,7 +1258,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 		gint ii;
 
 		/* get deleted messages from all active folders too */
-		folders = camel_object_bag_list (parent_store->folders);
+		folders = camel_store_dup_opened_folders (parent_store);
 		for (ii = 0; ii < folders->len; ii++) {
 			CamelFolder *opened_folder = CAMEL_FOLDER (folders->pdata[ii]);
 			CamelMapiFolder *mf;
@@ -1347,7 +1284,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 
 		if (status) {
 			camel_folder_freeze (folder);
-			mapi_summary_clear (folder->summary, TRUE);
+			mapi_summary_clear (folder_summary, TRUE);
 			camel_folder_thaw (folder);
 		} else if (mapi_error) {
 			if (!e_mapi_utils_propagate_cancelled_error (mapi_error, error))
@@ -1368,13 +1305,13 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 	}
 
 	changes = camel_folder_change_info_new ();
-	known_uids = camel_folder_summary_get_array (folder->summary);
+	folder_summary = camel_folder_get_folder_summary (folder);
+	known_uids = camel_folder_summary_get_array (folder_summary);
 
 	/*Collect UIDs of deleted messages.*/
 	for (i = 0; known_uids && i < known_uids->len; i++) {
-		info = camel_folder_summary_get (folder->summary, g_ptr_array_index (known_uids, i));
-		minfo = (CamelMapiMessageInfo *) info;
-		if (minfo && (minfo->info.flags & CAMEL_MESSAGE_DELETED)) {
+		info = camel_folder_summary_get (folder_summary, g_ptr_array_index (known_uids, i));
+		if (info && (camel_message_info_get_flags (info) & CAMEL_MESSAGE_DELETED) != 0) {
 			const gchar *uid = camel_message_info_get_uid (info);
 			mapi_id_t *mid = g_new0 (mapi_id_t, 1);
 
@@ -1390,7 +1327,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 			}
 			deleted_items_uid = g_slist_prepend (deleted_items_uid, (gpointer) uid);
 		}
-		camel_message_info_unref (info);
+		g_clear_object (&info);
 	}
 
 	camel_folder_summary_free_array (known_uids);
@@ -1415,11 +1352,11 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 		if (status) {
 			while (deleted_items_uid) {
 				const gchar *uid = (gchar *)deleted_items_uid->data;
-				camel_folder_summary_lock (folder->summary);
+				camel_folder_summary_lock (folder_summary);
 				camel_folder_change_info_remove_uid (changes, uid);
-				camel_folder_summary_remove_uid (folder->summary, uid);
+				camel_folder_summary_remove_uid (folder_summary, uid);
 				camel_data_cache_remove(mapi_folder->cache, "cache", uid, NULL);
-				camel_folder_summary_unlock (folder->summary);
+				camel_folder_summary_unlock (folder_summary);
 				deleted_items_uid = g_slist_next (deleted_items_uid);
 			}
 		}
@@ -1451,7 +1388,7 @@ mapi_folder_get_message_cached (CamelFolder *folder,
 
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 
-	if (!camel_folder_summary_check_uid (folder->summary, message_uid))
+	if (!camel_folder_summary_check_uid (camel_folder_get_folder_summary (folder), message_uid))
 		return NULL;
 
 	stream = camel_stream_mem_new ();
@@ -1515,7 +1452,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 	CamelMimeMessage *msg = NULL;
 	CamelMapiFolder *mapi_folder;
 	CamelMapiStore *mapi_store;
-	CamelMapiMessageInfo *mi = NULL;
+	CamelMessageInfo *mi;
 	CamelStore *parent_store;
 	mapi_id_t id_message;
 	EMapiConnection *conn;
@@ -1530,13 +1467,13 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 
 	/* see if it is there in cache */
 
-	mi = (CamelMapiMessageInfo *) camel_folder_summary_get (folder->summary, uid);
+	mi = camel_folder_summary_get (camel_folder_get_folder_summary (folder), uid);
 	if (mi == NULL) {
-		/* Translators: The first %s is replaced with a message ID,
-		   the second %s is replaced with a detailed error string */
 		g_set_error (
 			error, CAMEL_FOLDER_ERROR,
 			CAMEL_FOLDER_ERROR_INVALID_UID,
+			/* Translators: The first %s is replaced with a message ID,
+			   the second %s is replaced with a detailed error string */
 			_("Cannot get message %s: %s"), uid,
 			_("No such message"));
 		return NULL;
@@ -1544,7 +1481,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 
 	msg = mapi_folder_get_message_cached (folder, uid, cancellable);
 	if (msg != NULL) {
-		camel_message_info_unref (&mi->info);
+		g_clear_object (&mi);
 		return msg;
 	}
 
@@ -1553,7 +1490,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 			error, CAMEL_SERVICE_ERROR,
 			CAMEL_SERVICE_ERROR_UNAVAILABLE,
 			_("This message is not available in offline mode."));
-		camel_message_info_unref (&mi->info);
+		g_clear_object (&mi);
 		return NULL;
 	}
 
@@ -1571,13 +1508,15 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 				CAMEL_SERVICE_ERROR_INVALID,
 				_("Could not get message"));
 		}
-		camel_message_info_unref (&mi->info);
+		g_clear_object (&mi);
 		return NULL;
 	}
 
 	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
-	if (!conn)
+	if (!conn) {
+		g_clear_object (&mi);
 		return NULL;
+	}
 
 	e_mapi_util_mapi_id_from_string (uid, &id_message);
 
@@ -1604,13 +1543,13 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 				CAMEL_SERVICE_ERROR_INVALID,
 				_("Could not get message"));
 		}
-		camel_message_info_unref (&mi->info);
+		g_clear_object (&mi);
 		return NULL;
 	}
 
 	add_message_to_cache (mapi_folder, uid, &msg, cancellable);
 
-	camel_message_info_unref (&mi->info);
+	g_clear_object (&mi);
 
 	return msg;
 }
@@ -1632,9 +1571,9 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	CamelMapiStore *mapi_store;
 	CamelMapiFolder *mapi_folder;
 	CamelMessageInfo *info = NULL;
-	CamelMapiMessageInfo *mapi_info = NULL;
 	CamelStore *parent_store;
 	CamelFolderChangeInfo *changes = NULL;
+	CamelFolderSummary *folder_summary;
 	CamelServiceConnectionStatus status;
 	CamelService *service;
 	EMapiConnection *conn;
@@ -1651,6 +1590,7 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 
 	full_name = camel_folder_get_full_name (folder);
 	parent_store = camel_folder_get_parent_store (folder);
+	folder_summary = camel_folder_get_folder_summary (folder);
 
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
@@ -1672,18 +1612,17 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 
 	is_junk_folder = (mapi_folder->camel_folder_flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_JUNK;
 
-	camel_folder_summary_lock (folder->summary);
-	camel_folder_summary_prepare_fetch_all (folder->summary, NULL);
+	camel_folder_summary_lock (folder_summary);
+	camel_folder_summary_prepare_fetch_all (folder_summary, NULL);
 
-	known_uids = camel_folder_summary_get_array (folder->summary);
+	known_uids = camel_folder_summary_get_array (folder_summary);
 	for (i = 0; known_uids && i < known_uids->len; i++) {
-		info = camel_folder_summary_get (folder->summary, g_ptr_array_index (known_uids, i));
-		mapi_info = (CamelMapiMessageInfo *) info;
+		info = camel_folder_summary_get (folder_summary, g_ptr_array_index (known_uids, i));
 
-		if (mapi_info && (mapi_info->info.flags & CAMEL_MESSAGE_FOLDER_FLAGGED)) {
+		if (info && camel_message_info_get_folder_flagged (info)) {
 			const gchar *uid;
 			mapi_id_t *mid = g_new0 (mapi_id_t, 1); /* FIXME : */
-			guint32 flags;
+			guint32 flags, server_flags;
 			gboolean used = FALSE;
 
 			uid = camel_message_info_get_uid (info);
@@ -1691,17 +1630,18 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 
 			/* Why are we getting so much noise here :-/ */
 			if (!e_mapi_util_mapi_id_from_string (uid, mid)) {
-				camel_message_info_unref (info);
+				g_clear_object (&info);
 				g_free (mid);
 				continue;
 			}
 
-			mapi_utils_do_flags_diff (&diff, mapi_info->server_flags, mapi_info->info.flags);
-			mapi_utils_do_flags_diff (&unset_flags, flags, mapi_info->server_flags);
+			server_flags = camel_mapi_message_info_get_server_flags (CAMEL_MAPI_MESSAGE_INFO (info));
+			mapi_utils_do_flags_diff (&diff, server_flags, flags);
+			mapi_utils_do_flags_diff (&unset_flags, flags, server_flags);
 
-			diff.changed &= folder->permanent_flags;
+			diff.changed &= camel_folder_get_permanent_flags (folder);
 			if (!diff.changed) {
-				camel_message_info_unref (info);
+				g_clear_object (&info);
 				g_free (mid);
 				continue;
 			}
@@ -1728,15 +1668,14 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 			else
 				g_free (mid);
 
-			mapi_info->server_flags = mapi_info->info.flags;
+			camel_mapi_message_info_set_server_flags (CAMEL_MAPI_MESSAGE_INFO (info), camel_message_info_get_flags (info));
 		}
 
-		if (info)
-			camel_message_info_unref (info);
+		g_clear_object (&info);
 	}
 
 	camel_folder_summary_free_array (known_uids);
-	camel_folder_summary_unlock (folder->summary);
+	camel_folder_summary_unlock (folder_summary);
 
 	/*
 	   Sync up the READ changes before deleting the message.
@@ -1799,10 +1738,10 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 			changes = camel_folder_change_info_new ();
 		camel_folder_change_info_remove_uid (changes, deleted_msg_uid);
 
-		camel_folder_summary_lock (folder->summary);
-		camel_folder_summary_remove_uid (folder->summary, deleted_msg_uid);
+		camel_folder_summary_lock (folder_summary);
+		camel_folder_summary_remove_uid (folder_summary, deleted_msg_uid);
 		camel_data_cache_remove(mapi_folder->cache, "cache", deleted_msg_uid, NULL);
-		camel_folder_summary_unlock (folder->summary);
+		camel_folder_summary_unlock (folder_summary);
 
 		g_free (deleted_msg_uid);
 	}
@@ -1922,10 +1861,13 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 		g_clear_error (&mapi_error);
 		success = FALSE;
 	} else if (delete_originals) {
+		CamelFolderSummary *source_summary;
+
+		source_summary = camel_folder_get_folder_summary (source);
 		changes = camel_folder_change_info_new ();
 
 		for (i = 0; i < uids->len; i++) {
-			camel_folder_summary_remove_uid (source->summary, uids->pdata[i]);
+			camel_folder_summary_remove_uid (source_summary, uids->pdata[i]);
 			camel_folder_change_info_remove_uid (changes, uids->pdata[i]);
 		}
 		camel_folder_changed (source, changes);
@@ -2023,12 +1965,12 @@ camel_mapi_folder_class_init (CamelMapiFolderClass *class)
 	object_class->constructed = mapi_folder_constructed;
 
 	folder_class = CAMEL_FOLDER_CLASS (class);
+	folder_class->get_permanent_flags = mapi_folder_get_permanent_flags;
 	folder_class->rename = mapi_folder_rename;
 	folder_class->search_by_expression = mapi_folder_search_by_expression;
 	folder_class->cmp_uids = mapi_cmp_uids;
 	folder_class->search_by_uids = mapi_folder_search_by_uids;
 	folder_class->search_free = mapi_folder_search_free;
-	folder_class->set_message_flags = mapi_set_message_flags;
 	folder_class->append_message_sync = mapi_folder_append_message_sync;
 	folder_class->expunge_sync = mapi_folder_expunge_sync;
 	folder_class->get_message_sync = mapi_folder_get_message_sync;
@@ -2046,10 +1988,7 @@ camel_mapi_folder_init (CamelMapiFolder *mapi_folder)
 
 	mapi_folder->priv = G_TYPE_INSTANCE_GET_PRIVATE (mapi_folder, CAMEL_TYPE_MAPI_FOLDER, CamelMapiFolderPrivate);
 
-	folder->permanent_flags = CAMEL_MESSAGE_ANSWERED | CAMEL_MESSAGE_DELETED |
-		CAMEL_MESSAGE_DRAFT | CAMEL_MESSAGE_FLAGGED | CAMEL_MESSAGE_SEEN | CAMEL_MESSAGE_JUNK;
-
-	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY;
+	camel_folder_set_flags (folder, CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY);
 
 	g_mutex_init (&mapi_folder->priv->search_lock);
 
@@ -2064,19 +2003,18 @@ camel_mapi_folder_new (CamelStore *store,
 		       GError **error)
 {
 
-	CamelFolder	*folder = NULL;
+	CamelFolder *folder;
+	CamelFolderSummary *folder_summary;
 	CamelMapiFolder *mapi_folder;
-	CamelMapiStore  *mapi_store = (CamelMapiStore *) store;
-	CamelService    *service;
-	CamelSettings   *settings;
-
+	CamelMapiStore *mapi_store = (CamelMapiStore *) store;
+	CamelService *service;
+	CamelSettings *settings;
 	gchar *state_file;
 	const gchar *short_name;
 	CamelStoreInfo *si;
 	gboolean filter_inbox;
 
 	service = CAMEL_SERVICE (store);
-
 	settings = camel_service_ref_settings (service);
 
 	filter_inbox = camel_store_settings_get_filter_inbox (CAMEL_STORE_SETTINGS (settings));
@@ -2096,12 +2034,12 @@ camel_mapi_folder_new (CamelStore *store,
 		"parent-store", store,
 		NULL);
 
-	mapi_folder = CAMEL_MAPI_FOLDER(folder);
+	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 
-	folder->summary = camel_mapi_folder_summary_new (folder);
+	folder_summary = camel_mapi_folder_summary_new (folder);
 
-	if (!folder->summary) {
-		g_object_unref (CAMEL_OBJECT (folder));
+	if (!folder_summary) {
+		g_object_unref (folder);
 		g_set_error (
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Could not load summary for %s"),
@@ -2109,6 +2047,8 @@ camel_mapi_folder_new (CamelStore *store,
 		return NULL;
 	}
 
+	camel_folder_take_folder_summary (folder, folder_summary);
+
 	/* set/load persistent state */
 	state_file = g_build_filename (folder_dir, short_name, "cmeta", NULL);
 	camel_object_set_state_filename (CAMEL_OBJECT (folder), state_file);
@@ -2129,7 +2069,7 @@ camel_mapi_folder_new (CamelStore *store,
 		fi = camel_store_get_folder_info_sync (store, folder_name, 0, NULL, NULL);
 		if (fi) {
 			if ((fi->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_INBOX) {
-				folder->folder_flags |= CAMEL_FOLDER_FILTER_RECENT;
+				camel_folder_set_flags (folder, camel_folder_get_flags (folder) | CAMEL_FOLDER_FILTER_RECENT);
 			}
 
 			camel_folder_info_free (fi);
@@ -2145,6 +2085,7 @@ camel_mapi_folder_new (CamelStore *store,
 	si = camel_store_summary_path (mapi_store->summary, folder_name);
 	if (si) {
 		CamelMapiStoreInfo *msi = (CamelMapiStoreInfo *) si;
+		guint32 add_folder_flags = 0;
 
 		mapi_folder->mapi_folder_flags = msi->mapi_folder_flags;
 		mapi_folder->camel_folder_flags = msi->camel_folder_flags;
@@ -2156,17 +2097,19 @@ camel_mapi_folder_new (CamelStore *store,
 		}
 
 		if ((si->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_TRASH)
-			folder->folder_flags |= CAMEL_FOLDER_IS_TRASH;
+			add_folder_flags |= CAMEL_FOLDER_IS_TRASH;
 		else if ((si->flags & CAMEL_FOLDER_TYPE_MASK) == CAMEL_FOLDER_TYPE_JUNK)
-			folder->folder_flags |= CAMEL_FOLDER_IS_JUNK;
+			add_folder_flags |= CAMEL_FOLDER_IS_JUNK;
 		camel_store_summary_info_unref (mapi_store->summary, si);
+
+		camel_folder_set_flags (folder, camel_folder_get_flags (folder) | add_folder_flags);
 	} else {
 		g_warning ("%s: cannot find '%s' in known folders", G_STRFUNC, folder_name);
 	}
 
 	camel_store_summary_connect_folder_summary (
 		((CamelMapiStore *) store)->summary,
-		folder_name, folder->summary);
+		folder_name, folder_summary);
 
 	/* sanity checking */
 	if ((mapi_folder->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_FOREIGN) != 0)
